@@ -18,6 +18,7 @@ BEGIN
     use warnings::register;
     use parent qw( Apache2::SSI::Common );
     use Apache2::SSI::Finfo;
+    use File::Spec ();
     use Scalar::Util ();
     use URI::file ();
     if( $ENV{MOD_PERL} )
@@ -39,6 +40,7 @@ BEGIN
         fallback => 1,
     );
     our $VERSION = 'v0.1.0';
+    our $DIR_SEP = $Apache2::SSI::Common::DIR_SEP;
 };
 
 sub init
@@ -63,15 +65,15 @@ sub init
         }
         else
         {
-            my @segments = split( '/', $self->{base_file}, -1 );
+            my @segments = split( "\Q${DIR_SEP}\E", $self->{base_file}, -1 );
             pop( @segments );
-            $base_dir = join( '/', @segments );
+            $base_dir = join( $DIR_SEP, @segments );
         }
         $self->{base_dir} = $base_dir;
     }
     elsif( !length( $self->{base_dir} ) )
     {
-        $base_dir = URI->new( URI::file->cwd )->file;
+        $base_dir = URI->new( URI::file->cwd )->file( $^O );
         $self->{base_dir} = $base_dir;
     }
     $self->filename( $file ) || return;
@@ -148,7 +150,7 @@ sub filename
             {
                 $self->message( 3, "File is not found." );
                 $self->code( 404 );
-                $newfile = $self->collapse_dots( $newfile );
+                $newfile = $self->collapse_dots( $newfile, { separator => $DIR_SEP });
                 ## We don't pass it the Apache2::RequestRec object, because it would trigger a fatal error since the file does not exist. Instead, we use the api without Apache2::RequestRec which is more tolerant
                 ## We do this so the user can call our object $file->finfo->filetype == Apache2::SSI::Finfo::FILETYPE_NOFILE
                 $self->{finfo} = Apache2::SSI::Finfo->new( $newfile );
@@ -165,11 +167,14 @@ sub filename
         if( defined( $newfile ) )
         {
             my $base_dir = $self->base_dir;
-            $base_dir .= '/' unless( substr( $base_dir, -1, 1 ) eq '/' );
-            $self->message( 3, "New file path provided is: '$newfile' and base directory is '$base_dir'" );
-            $newfile = URI::file->new( $newfile )->abs( $base_dir )->file;
+            $base_dir .= $DIR_SEP unless( substr( $base_dir, -length( $DIR_SEP ), length( $DIR_SEP ) ) eq $DIR_SEP );
+            $self->message( 3, "New file path provided is: '$newfile' and base directory is '$base_dir' and directory separator is '$DIR_SEP'" );
+            ## If we provide a string for the abs() method it works on Unix, but not on Windows
+            ## By providing an object, we make it work
+            $newfile = URI::file->new( $newfile )->abs( URI::file->new( $base_dir ) )->file( $^O );
             $self->message( 3, "Getting the new file real path: '$newfile'" );
-            $self->{filename} = $self->collapse_dots( $newfile );
+            $self->{filename} = $self->collapse_dots( $newfile, { separator => $DIR_SEP })->file( $^O );
+            $self->message( 3, "Filename after dot collapsing is: '$self->{filename}'" );
             $self->finfo( $newfile );
             my $finfo = $self->finfo;
             $self->message( 3, "finfo is '", overload::StrVal( $finfo ), "'." );
@@ -206,7 +211,7 @@ sub finfo
     
     if( defined( $newfile ) )
     {
-        $self->{finfo} = Apache2::SSI::Finfo->new( $newfile, ( $r ? ( apache_request => $r ) : () ) );
+        $self->{finfo} = Apache2::SSI::Finfo->new( $newfile, ( $r ? ( apache_request => $r ) : () ), debug => $self->debug );
         $self->message( 3, "finfo object is now '", overload::StrVal( $self->{finfo} ), "'" );
         $self->message( 3, "Error occurred: ", Apache2::SSI::Finfo->error ) if( !$self->{finfo} );
         return( $self->pass_error( Apache2::SSI::Finfo->error ) ) if( !$self->{finfo} );
@@ -220,12 +225,19 @@ sub parent
     my $self = shift( @_ );
     my $r = $self->apache_request;
     ## I deliberately did not do split( '/', $path, -1 ) so that if there is a trailing '/', it will not be counted
-    my @segments = split( '/', $self->filename, -1 );
+    ## 2021-03-27: Was working well, but only on Unix systems...
+    ## my @segments = split( '/', $self->filename, -1 );
+    my( $vol, $parent, $file ) = File::Spec->splitpath( $self->filename );
+    $vol //= '';
+    $file //= '';
+    $self->message( 3, "Filename is '", $self->filename, "', volume is '$vol', parent '$parent' and file is '$file'." );
+    my @segments = File::Spec->splitpath( File::Spec->catfile( $parent, $file ) );
     ## $self->message( 3, "Path segments are: ", sub{ $self->dump( \@segments )} );
     pop( @segments );
     return( $self ) if( !scalar( @segments ) );
-    $self->message( 3, "Creating new object with document uri '", join( '/', @segments ), "'." );
-    return( $self->new( join( '/', @segments ), ( $r ? ( apache_request => $r ) : () ) ) );
+    $self->message( 3, "Creating new object with document uri '", $vol . File::Spec->catdir( @segments ), "'." );
+    ## return( $self->new( join( '/', @segments ), ( $r ? ( apache_request => $r ) : () ) ) );
+    return( $self->new( $vol . File::Spec->catdir( @segments ), ( $r ? ( apache_request => $r ) : () ) ) );
 }
 
 sub _make_abs
@@ -238,11 +250,12 @@ sub _make_abs
         $self->message( 3, "Setting $field to '$this'." );
         if( Scalar::Util::blessed( $this ) && $this->isa( 'URI::file' ) )
         {
-            $this = URI->new_abs( $this )->file;
+            $this = URI->new_abs( $this )->file( $^O );
         }
-        elsif( substr( $this, 0, 1 ) ne '/' )
+        ## elsif( substr( $this, 0, 1 ) ne '/' )
+        elsif( !File::Spec->file_name_is_absolute( $this ) )
         {
-            $this = URI::file->new_abs( $this )->file;
+            $this = URI::file->new_abs( $this )->file( $^O );
         }
         $self->message( 3, "$field is now '$this'" );
         $self->{ $field } = $this;
